/*
CREATE DATABASE TestApollo

USE TestApollo

IF OBJECT_ID('FactResellerSales') IS NOT NULL
	DROP TABLE FactResellerSales

CREATE TABLE [dbo].[FactResellerSales]( 
    [ProductKey] [int] NOT NULL, 
    [OrderDateKey] [int] NOT NULL, 
    [DueDateKey] [int] NOT NULL, 
    [ShipDateKey] [int] NOT NULL, 
    [CustomerKey] [int] NOT NULL, 
    [EmployeeKey] [int] NOT NULL, 
    [PromotionKey] [int] NOT NULL, 
    [CurrencyKey] [int] NOT NULL, 
    [SalesTerritoryKey] [int] NOT NULL, 
    [SalesOrderNumber] [nvarchar](20) NOT NULL, 
    [SalesOrderLineNumber] [tinyint] NOT NULL, 
    [RevisionNumber] [tinyint] NULL, 
    [OrderQuantity] [smallint] NULL, 
    [UnitPrice] [money] NULL, 
    [ExtendedAmount] [money] NULL, 
    [UnitPriceDiscountPct] [float] NULL, 
    [DiscountAmount] [float] NULL, 
    [ProductStandardCost] [money] NULL, 
    [TotalProductCost] [money] NULL, 
    [SalesAmount] [money] NULL, 
    [TaxAmt] [money] NULL, 
    [Freight] [money] NULL, 
    [CarrierTrackingNumber] [nvarchar](25) NULL, 
    [CustomerPONumber] [nvarchar](25) NULL,
    OrderDate [datetime] NULL,
    DueDate [datetime] NULL,
    ShipDate [datetime] NULL
)



CREATE UNIQUE CLUSTERED INDEX ix_FactResellerSales_Keys ON FactResellerSales
	(CustomerKey,OrderDateKey,ProductKey)

--load rows at this point

SELECT ps.row_count
FROM sys.dm_db_partition_stats ps
	JOIN sys.partitions sp ON ps.partition_id = sp.partition_id
WHERE sp.object_id = OBJECT_ID('FactResellerSales')
	AND sp.index_id = 1

ALTER INDEX ix_FactResellerSales_Keys ON FactResellerSales REBUILD WITH (DATA_COMPRESSION = NONE, FILLFACTOR = 30)
ALTER INDEX ix_FactResellerSales_Keys ON FactResellerSales REBUILD WITH (DATA_COMPRESSION = NONE, FILLFACTOR = 100)
--1:13:24 with 12,020,821 rows on i5ASUS - 25 minutes to scan
--0:04:38 with  6,000,000 rows on i5ASUS - 38 seconds to scan (with clear buffers)
	--0:08:33 to create columnstore index
--0:00:58 with 4,000,000 rows on i5ASUS
	--0:01:25 to create columnstore index

SELECT * FROM sys.dm_db_index_physical_stats(DB_ID(),OBJECT_ID('FactResellerSales'),1,NULL,NULL)

DROP INDEX FactResellerSales.csix_FactResellerSales

CREATE COLUMNSTORE INDEX csix_FactResellerSales
ON FactResellerSales
(			[ProductKey]
           ,[OrderDateKey]
           ,[DueDateKey]
           ,[ShipDateKey]
           ,[CustomerKey]
           ,[EmployeeKey]
           ,[PromotionKey]
           ,[CurrencyKey]
           ,[SalesTerritoryKey]
				,[SalesOrderNumber]
           ,[SalesOrderLineNumber]
           ,[RevisionNumber]
           ,[OrderQuantity]
           ,[UnitPrice]
           ,[ExtendedAmount]
           ,[UnitPriceDiscountPct]
           ,[DiscountAmount]
           ,[ProductStandardCost]
           ,[TotalProductCost]
           ,[SalesAmount]
           ,[TaxAmt]
           ,[Freight]
				,[CarrierTrackingNumber]
				,[CustomerPONumber]
           ,[OrderDate]
           ,[DueDate]
           ,[ShipDate]
)
*/




SET NOCOUNT ON

DECLARE @RecordCount int = 0
DECLARE @FailCount int = 0

DECLARE @ProductKey int
DECLARE @OrderDateKey int
DECLARE @DueDateKey int
DECLARE @ShipDateKey int
DECLARE @CustomerKey int
DECLARE @EmployeeKey int
DECLARE @PromotionKey int
DECLARE @CurrencyKey int
DECLARE @SalesTerritoryKey int
DECLARE @SalesOrderNumber nvarchar(20)
DECLARE @SalesOrderLineNumber tinyint
DECLARE @RevisionNumber tinyint
DECLARE @OrderQuantity smallint
DECLARE @UnitPrice money
DECLARE @ExtendedAmount money
DECLARE @UnitPriceDiscountPct float
DECLARE @DiscountAmount float
DECLARE @ProductStandardCost money
DECLARE @TotalProductCost money
DECLARE @SalesAmount money
DECLARE @TaxAmt money
DECLARE @Freight money
DECLARE @CarrierTrackingNumber nvarchar(25)
DECLARE @CustomerPONumber nvarchar(25)
DECLARE @OrderDate datetime
DECLARE @DueDate datetime
DECLARE @ShipDate datetime

DECLARE @StartTime datetime = GETDATE()
DECLARE @Text nvarchar(1000)
DECLARE @Total int = (SELECT 10000000 - COUNT(*) FROM FactResellerSales)
DECLARE @Rate float
DECLARE @SecsRemain int

WHILE @RecordCount < 6989768
BEGIN

SET @ProductKey = RAND(CHECKSUM(NEWID())) * 1000
SET @OrderDateKey = RAND(CHECKSUM(NEWID())) * 10000
SET @DueDateKey = RAND(CHECKSUM(NEWID())) * 10000
SET @ShipDateKey = RAND(CHECKSUM(NEWID())) * 10000
SET @CustomerKey = RAND(CHECKSUM(NEWID())) * 100
SET @EmployeeKey = RAND(CHECKSUM(NEWID())) * 10
SET @PromotionKey = RAND(CHECKSUM(NEWID())) * 100
SET @CurrencyKey = RAND(CHECKSUM(NEWID())) * 5
SET @SalesTerritoryKey = RAND(CHECKSUM(NEWID())) * 100
SET @SalesOrderNumber = SUBSTRING(REPLACE(CAST(NEWID() AS varchar(100)),'-',''),1,10)
SET @SalesOrderLineNumber = 1
SET @RevisionNumber = 1
SET @OrderQuantity = RAND(CHECKSUM(NEWID())) * 100 + 1
SET @UnitPrice = 50.0 + RAND(CHECKSUM(NEWID())) * 1000.0
SET @ExtendedAmount = 50.0 + RAND(CHECKSUM(NEWID())) * 1000.0
SET @UnitPriceDiscountPct = 0.00
SET @DiscountAmount = 0.00
SET @ProductStandardCost = 0.00
SET @TotalProductCost = 0.00
SET @SalesAmount = 0.00
SET @TaxAmt = 0.00
SET @Freight = 0.00
SET @CarrierTrackingNumber = SUBSTRING(REPLACE(CAST(NEWID() AS varchar(100)),'-',''),1,20)
SET @CustomerPONumber = SUBSTRING(REPLACE(CAST(NEWID() AS varchar(100)),'-',''),1,8)
SET @OrderDate = '2011-01-01'
SET @DueDate = '2011-01-01'
SET @ShipDate = '2011-01-01'



IF NOT EXISTS(SELECT 1 FROM FactResellerSales WHERE CustomerKey = @CustomerKey AND OrderDateKey = @OrderDateKey AND ProductKey = @ProductKey)
BEGIN
INSERT INTO [dbo].[FactResellerSales]
           ([ProductKey]
           ,[OrderDateKey]
           ,[DueDateKey]
           ,[ShipDateKey]
           ,[CustomerKey]
           ,[EmployeeKey]
           ,[PromotionKey]
           ,[CurrencyKey]
           ,[SalesTerritoryKey]
           ,[SalesOrderNumber]
           ,[SalesOrderLineNumber]
           ,[RevisionNumber]
           ,[OrderQuantity]
           ,[UnitPrice]
           ,[ExtendedAmount]
           ,[UnitPriceDiscountPct]
           ,[DiscountAmount]
           ,[ProductStandardCost]
           ,[TotalProductCost]
           ,[SalesAmount]
           ,[TaxAmt]
           ,[Freight]
           ,[CarrierTrackingNumber]
           ,[CustomerPONumber]
           ,[OrderDate]
           ,[DueDate]
           ,[ShipDate])
     VALUES
           (@ProductKey       -- int
           ,@OrderDateKey       -- int
           ,@DueDateKey       -- int
           ,@ShipDateKey       -- int
           ,@CustomerKey       -- int
           ,@EmployeeKey       -- int
           ,@PromotionKey       -- int
           ,@CurrencyKey       -- int
           ,@SalesTerritoryKey       -- int
           ,@SalesOrderNumber       -- nvarchar(20)
           ,@SalesOrderLineNumber       -- tinyint
           ,@RevisionNumber       -- tinyint
           ,@OrderQuantity       -- smallint
           ,@UnitPrice       -- money
           ,@ExtendedAmount       -- money
           ,@UnitPriceDiscountPct       -- float
           ,@DiscountAmount       -- float
           ,@ProductStandardCost       -- money
           ,@TotalProductCost       -- money
           ,@SalesAmount       -- money
           ,@TaxAmt       -- money
           ,@Freight       -- money
           ,@CarrierTrackingNumber       -- nvarchar(25)
           ,@CustomerPONumber       -- nvarchar(25)
           ,@OrderDate       -- datetime
           ,@DueDate       -- datetime
           ,@ShipDate       -- datetime
		  )
SET @RecordCount += 1
END
ELSE
BEGIN
	SET @FailCount += 1
END

IF @RecordCount % 10000 = 0
BEGIN
	SET @Rate = 10000.0 / (DATEDIFF(millisecond,@StartTime,GETDATE()) / 1000.0)
	SET @SecsRemain = (@total - @RecordCount) / @Rate

	SET @Text = CONVERT(varchar,GETDATE(),121) 
		+ ': Inserted ' + CAST(@RecordCount AS varchar) + ' with a failcount of ' + CAST(@FailCount AS varchar) + ' ' 
		+ CAST(CAST(@Rate AS decimal(19,2)) AS varchar)
		+ ' records per second '
		+ CAST(CAST(@SecsRemain / 3600.0 AS decimal(19,2)) AS varchar) + ' hours remain estimated completion: '
		+ CONVERT(varchar,DATEADD(second,@SecsRemain,GETDATE()),121)
	IF @Text IS NULL SET @Text = 'Error'
	RAISERROR(@Text,0,0) WITH NOWAIT

	SET @StartTime = GETDATE()
END

END

RAISERROR('Completed inserting %i with a failcount of %i',0,0,@RecordCount,@FailCount)