/*	SQL 2012 What's New
	Michael DeFehr
	http://mikedefehr.com
	mike@defehr.com
*/


USE master
GO

SET STATISTICS IO OFF
SET STATISTICS TIME OFF

--new bits
SELECT @@VERSION

intellisense
SELECT * FROM sys.prod 

USE master
GO
IF EXISTS(SELECT * FROM sys.databases WHERE name = 'TestDB')
	DROP DATABASE TestDB
CREATE DATABASE TestDB


--User-Defined Server Roles
-- example from Ashish Kumar Mehta on mssqltips.com
IF EXISTS(SELECT * FROM sys.server_principals WHERE name = 'LevelOneDBAs') 
BEGIN
	ALTER SERVER ROLE LevelOneDBAs DROP MEMBER L1DBA
	DROP SERVER ROLE [LevelOneDBAs]
END

CREATE SERVER ROLE [LevelOneDBAs] AUTHORIZATION [sa]

GRANT VIEW SERVER STATE TO [LevelOneDBAs]

IF EXISTS (SELECT * FROM sys.server_principals WHERE name = 'L1DBA') DROP LOGIN L1DBA
CREATE LOGIN [L1DBA] WITH PASSWORD = 'P@ssword01'

IF EXISTS (SELECT * FROM sys.server_principals WHERE name = 'Nobody') DROP LOGIN Nobody
CREATE LOGIN [Nobody] WITH PASSWORD = 'P@ssword01'

ALTER SERVER ROLE [LevelOneDBAs] ADD MEMBER [L1DBA]

--Login as L1DBA

BACKUP DATABASE TestDB TO DISK = 'TestDB.BAK'

SHUTDOWN

SELECT * FROM sys.dm_exec_requests

SELECT * FROM sys.dm_os_waiting_tasks




--Contained Databases - Database Authentication
USE master
GO

sp_configure 'show advanced', 1
RECONFIGURE
GO

sp_configure 'contained database authentication',1
RECONFIGURE
GO

IF EXISTS(SELECT * FROM sys.databases WHERE name = 'ContainedDB') 
	DROP DATABASE ContainedDB
GO
IF EXISTS(SELECT * FROM sys.databases WHERE name = 'ContainedDB2') 
	DROP DATABASE ContainedDB2
GO
CREATE DATABASE ContainedDB
CONTAINMENT = PARTIAL
GO

USE ContainedDB
GO
CREATE ROLE AppRole
GRANT VIEW DEFINITION TO AppRole
GO

CREATE TABLE ATableInContainedDB (ID int PRIMARY KEY, Data varchar(255))
GRANT SELECT ON ATableInContainedDB TO AppRole
GO
-- to create a contained SQL User
CREATE USER AppUser WITH PASSWORD = 'P@ssword01' 
ALTER ROLE AppRole ADD MEMBER AppUser
GO

-- to create a contained Windows User
CREATE USER [MDD-ASUS\TestUser]
ALTER ROLE AppRole ADD MEMBER [MDD-ASUS\TestUser]
GO

CREATE DATABASE ContainedDB2
CONTAINMENT = PARTIAL
GO

USE ContainedDB2
GO
CREATE ROLE AppRole
GRANT VIEW DEFINITION TO AppRole
GO

CREATE TABLE ATableInTheSecondContainedDB (ID int PRIMARY KEY, Data varchar(255))
GRANT SELECT ON ATableInTheSecondContainedDB TO AppRole
GO

-- to create a contained SQL User
CREATE USER AppUser WITH PASSWORD = 'P@ssword01'
ALTER ROLE AppRole ADD MEMBER AppUser
GO

-- to create a contained Windows User
CREATE USER [MDD-ASUS\TestUser]
ALTER ROLE AppRole ADD MEMBER [MDD-ASUS\TestUser]
GO

--/demo






--FileStream / FileTable
sp_configure 'filestream access level', 2
RECONFIGURE

--Filestream is enabled in configuration manager - SQL Service - FILESTREAM tab

--filestream access level = 2
USE TestDB
GO

SELECT * FROM sys.sysfiles

ALTER DATABASE TestDB ADD FILEGROUP FGFileStream CONTAINS FILESTREAM 

ALTER DATABASE TestDB ADD FILE
(	NAME = FGFileStream,
	FILENAME = 'D:\S2012DEV\Data\TestDBFS'	)
TO FILEGROUP FGFileStream


SELECT DB_NAME(database_id), non_transacted_access, non_transacted_access_desc, directory_name
FROM sys.database_filestream_options
WHERE database_id = DB_ID()

ALTER DATABASE TestDB
	SET FILESTREAM ( NON_TRANSACTED_ACCESS = FULL, DIRECTORY_NAME = 'TestDB_FS'	)

CREATE TABLE DocumentStore AS FileTable
WITH (
	FileTable_Directory = 'DocumentTable',
	FileTable_Collate_FileName = database_default,
	FileTable_Primary_Key_Constraint_Name = PK_DocumentStore
	)

SELECT *
FROM DocumentStore


INSERT DocumentStore (name, file_stream)
VALUES ('TestFile.txt',CAST('Here is some text' AS varbinary(max)))

DELETE FROM documentstore WHERE name = 'TestFile.txt'

SELECT file_stream, CAST(file_stream AS varchar(1000))
FROM DocumentStore
WHERE name = 'TestFile.txt'


--Fulltext stuff

--install PDF iFilter
EXEC sp_fulltext_service 'load_os_resources', 1
EXEC sp_fulltext_service 'verify_signature',0
--restart SQL
EXEC sp_fulltext_service 'update_languages'
EXEC sp_fulltext_service 'restart_all_fdhosts'

SELECT * FROM sys.fulltext_document_types WHERE document_type = '.pdf'

USE TestDB
GO
/*
DROP FULLTEXT INDEX ON DocumentStore
DROP FULLTEXT CATALOG TestDB_Catalog
*/
CREATE FULLTEXT CATALOG TestDB_Catalog AS DEFAULT

SELECT * FROM sys.fulltext_languages

SELECT * FROM Documentstore

CREATE FULLTEXT INDEX ON DocumentStore
(	name LANGUAGE 1033,
	file_stream TYPE COLUMN file_type LANGUAGE 1033
)
	KEY INDEX PK_DocumentStore
	ON TestDB_Catalog
	WITH 
		CHANGE_TRACKING AUTO,
		STOPLIST = SYSTEM

SELECT name
FROM DocumentStore
WHERE
CONTAINS(file_stream,'appliance')

--/demo




--Columnstore Index
USE TestApollo
GO
IF EXISTS(SELECT * FROM sys.indexes WHERE [object_id] = OBJECT_ID('FactResellerSales') AND name = 'ix_FactResellerSales_SalesTerritoryKey')
	DROP INDEX FactResellerSales.ix_FactResellerSales_SalesTerritoryKey
GO

SELECT COUNT(*) FROM FactResellerSales

SET STATISTICS IO ON
SET STATISTICS TIME ON




--12 reads? 2M rows - 8KB per page 8*12*1024 = 98304 bytes / 2M = 0.49 bytes per row

SELECT TOP 10 * FROM FactResellerSales



DBCC DROPCLEANBUFFERS

SELECT SalesTerritoryKey, SUM(ExtendedAmount)
FROM FactResellerSales
GROUP BY SalesTerritoryKey

/*

SalesTerritoryKey int (4 bytes)
ExtendedAmount money (8 bytes)

clustering key - [CustomerKey] int (4 bytes)
				 [OrderDateKey] int (4 bytes)
				 [ProductKey] int (4 bytes)

24 * 2000000 / 1024 / 8 = 5859 pages without overhead

*/

DBCC DROPCLEANBUFFERS

SELECT SalesTerritoryKey, SUM(ExtendedAmount)
FROM FactResellerSales
GROUP BY SalesTerritoryKey

SELECT SalesTerritoryKey, SUM(ExtendedAmount)
FROM FactResellerSales WITH (INDEX(ix_FactResellerSales_SalesTerritoryKey))
GROUP BY SalesTerritoryKey

SELECT SalesTerritoryKey, SUM(ExtendedAmount)
FROM FactResellerSales WITH (INDEX(ix_FactResellerSales_Keys))
GROUP BY SalesTerritoryKey


CREATE INDEX ix_FactResellerSales_SalesTerritoryKey ON FactResellerSales (SalesTerritoryKey)
	INCLUDE (ExtendedAmount)
GO



SELECT CustomerKey, SUM(ExtendedAmount)
FROM FactResellerSales
GROUP BY CustomerKey

SELECT CustomerKey, SUM(OrderQuantity)
FROM FactResellerSales
GROUP BY CustomerKey

SELECT index_id FROM sys.indexes WHERE name = 'ix_FactResellerSales_SalesTerritoryKey'

SELECT 'Clustered', page_count * 8 / 1024.0  FROM sys.dm_db_index_physical_stats(DB_ID('TestApollo'),OBJECT_ID('FactResellerSales'),1,NULL,NULL)
SELECT 'NonClustered', page_count * 8 / 1024.0  FROM sys.dm_db_index_physical_stats(DB_ID('TestApollo'),OBJECT_ID('FactResellerSales'),6,NULL,NULL)
SELECT 'Columnstore', SUM(on_disk_size_MB) AS TotalSizeInMB
FROM
(
   (SELECT SUM(css.on_disk_size)/(1024.0*1024.0) on_disk_size_MB
    FROM sys.indexes AS i
    JOIN sys.partitions AS p
        ON i.object_id = p.object_id 
    JOIN sys.column_store_segments AS css
        ON css.hobt_id = p.hobt_id
    WHERE i.object_id = object_id('FactResellerSales') 
    AND i.type_desc = 'NONCLUSTERED COLUMNSTORE') 
  UNION ALL
   (SELECT SUM(csd.on_disk_size)/(1024.0*1024.0) on_disk_size_MB
    FROM sys.indexes AS i
    JOIN sys.partitions AS p
        ON i.object_id = p.object_id 
    JOIN sys.column_store_dictionaries AS csd
        ON csd.hobt_id = p.hobt_id
    WHERE i.object_id = object_id('FactResellerSales') 
    AND i.type_desc = 'NONCLUSTERED COLUMNSTORE') 
) AS SegmentsPlusDictionary
--With Page compression - 169.047 MB

--great, right? remember the downside
UPDATE FactResellerSales SET DiscountAmount = 5
WHERE CustomerKey = 6 AND OrderDateKey = 10 AND ProductKey = 741



--/demo







--even simpler paging
SET STATISTICS IO OFF
SET STATISTICS TIME OFF
USE TestDB
GO
IF OBJECT_ID('QueryResults') IS NOT NULL DROP TABLE QueryResults
GO
CREATE TABLE QueryResults (
	ItemIndex int IDENTITY(1683,3) PRIMARY KEY,
	SomeValue uniqueidentifier DEFAULT NEWID(),
	SomeOtherValue uniqueidentifier DEFAULT NEWID()
)
GO

INSERT QueryResults default values
GO 200

SELECT *
FROM QueryResults

DECLARE @Page int = 3;
DECLARE @PageSize int = 20;

WITH cte AS
(
	SELECT *, ROW_NUMBER() OVER (ORDER BY ItemIndex) AS rn
	FROM QueryResults
)
SELECT *
FROM cte
WHERE rn BETWEEN @Page * @PageSize + 1 AND (@Page + 1) * @PageSize;
GO

DECLARE @Page int = 3;
DECLARE @PageSize int = 20;

SELECT *
FROM QueryResults
ORDER BY ItemIndex
OFFSET @Page * @PageSize ROWS
FETCH NEXT @PageSize ROWS ONLY;


--Sequence Generators
USE TestDB
GO
CREATE SEQUENCE IDSequence AS int
	START WITH 1 INCREMENT BY 1;
GO


SELECT NEXT VALUE FOR IDSequence
GO

CREATE TABLE ImportantStuff (
	ID int,
	Data uniqueidentifier
)
GO

INSERT ImportantStuff
	VALUES (NEXT VALUE FOR IDSequence, NEWID())


SELECT * FROM ImportantStuff


CREATE TABLE Table1 (
	ID int PRIMARY KEY DEFAULT NEXT VALUE FOR IDSequence,
	Data uniqueidentifier DEFAULT NEWID()
)

CREATE TABLE Table2 (
	ID int PRIMARY KEY DEFAULT NEXT VALUE FOR IDSequence,
	Data uniqueidentifier DEFAULT NEWID()
)


INSERT Table1 DEFAULT VALUES
INSERT Table1 DEFAULT VALUES
INSERT Table2 DEFAULT VALUES
INSERT Table1 DEFAULT VALUES
INSERT Table2 DEFAULT VALUES
INSERT Table2 DEFAULT VALUES
INSERT Table2 DEFAULT VALUES
INSERT Table1 DEFAULT VALUES
INSERT Table1 DEFAULT VALUES
INSERT Table1 DEFAULT VALUES

SELECT * FROM Table1
SELECT * FROM Table2




SELECT	NEXT VALUE FOR IDSequence,
		Data
FROM Table1


SELECT	NEXT VALUE FOR IDSequence OVER(ORDER BY Data),
		Data
FROM Table1

DECLARE @FirstValue sql_variant, @LastValue sql_variant
EXEC sp_sequence_get_range 'IDSequence', 10, @FirstValue OUTPUT, @LastValue OUTPUT
SELECT @FirstValue, @LastValue

SELECT	NEXT VALUE FOR IDSequence


--Window functions
USE TestDB
GO

IF OBJECT_ID('SalesData') IS NOT NULL DROP TABLE SalesData;
GO
CREATE TABLE SalesData (
	SalesDate Date PRIMARY KEY,
	SalesAmount money
)
GO

DECLARE @InsDate Date
WHILE (SELECT COUNT(*) FROM SalesData) < 50
BEGIN
	SET @InsDate = DATEADD(day,RAND(CHECKSUM(NEWID())) * 300,'2011-01-01')
	IF NOT EXISTS(SELECT 1 FROM SalesData WHERE SalesDate = @InsDate)
		INSERT SalesData VALUES (@InsDate,RAND(CHECKSUM(NEWID())) * 10000 + 1000)
END;

SELECT *
FROM SalesData;

SELECT *, (SELECT SUM(SalesAmount) FROM SalesData WHERE SalesDate <= sd.SalesDate) AS RunningSum
FROM SalesData sd
ORDER BY SalesDate;

WITH sd AS
(
	SELECT *, ROW_NUMBER() OVER (ORDER BY SalesDate) AS rn
	FROM Salesdata
),
cte AS
(
	SELECT *, SalesAmount AS RunningSum
	FROM sd
	WHERE rn = 1

	UNION ALL

	SELECT sd.*, sd.SalesAmount + cte.RunningSum
	FROM sd
		JOIN cte ON sd.rn = cte.rn + 1
)
SELECT * FROM cte;

SELECT	*, 
		SUM(SalesAmount) OVER (ORDER BY SalesDate RANGE BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) AS RunningSum
FROM SalesData;



SELECT	*,
		DATEDIFF(day, LAG(SalesDate) OVER(ORDER BY SalesDate), SalesDate) AS NumDays
FROM SalesData;

SELECT *,
		AVG(SalesAmount) OVER (ORDER BY SalesDate ROWS BETWEEN 4 PRECEDING AND CURRENT ROW) AS RollingAverage
FROM SalesData;


--TRY_CONVERT/TRY_CAST/IIF

CREATE TABLE AddressData (
	StreetAddress varchar(255),
	City varchar(100),
	StateProv char(2),
	PostalZip varchar(15),
	Country varchar(255)
)

INSERT AddressData VALUES 
('123 Elm St.','Springfield','ND','12344','USA'),
('321 Martin Luther King Blvd.','Chicago','IL','56784','USA'),
('456 Washington Rd.','Seattle','WA','87462','USA'),
('123 Portage Ave','Winnipeg','MB','R2K 1B4','Canada')


SELECT *
FROM AddressData
WHERE CAST(PostalZip AS int) > 50000

SELECT *
FROM AddressData
WHERE TRY_CAST(PostalZip AS int) > 50000


SELECT *, TRY_CAST(PostalZip AS int)
FROM AddressData


SELECT *, IIF(City = 'Winnipeg','Close to home!','A ways to go...')
FROM AddressData


--Date Functions

SELECT EOMONTH(GETDATE())

SELECT DATEFROMPARTS(2011,11,21)


--Metadata discovery

DECLARE @SQL nvarchar(1000) = 'SELECT * FROM AddressData'
EXEC sp_describe_first_result_set @SQL
GO

DECLARE @SQL nvarchar(1000) = 'SELECT * FROM AddressData WHERE StateProv = @1'
EXEC sp_executesql @SQL, N'@1 char(2)', @1 = 'WA'
GO

DECLARE @SQL nvarchar(1000) = 'SELECT * FROM AddressData WHERE StateProv = @1 AND City = @2'
DECLARE @Prm nvarchar(1000) = N'@1 char(2)'
EXEC sp_executesql @SQL, @Prm, @1 = 'WA'
GO

DECLARE @SQL nvarchar(1000) = 'SELECT * FROM AddressData WHERE StateProv = @1 AND City = @2'
DECLARE @Prm nvarchar(1000) = N'@1 char(2)'
exec sp_describe_undeclared_parameters @SQL, @Prm


--/demo


